# label.tcl --

# A library of label routines.

# Version   : 0.0.1
# Author    : Mark G. Saye
# Email     : markgsaye@yahoo.com
# Copyright : Copyright (C) 2003
# Date      : April 20, 2003

# See the file "LICENSE.txt" or "LICENSE.html" for information on usage
# and distribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# ======================================================================

# 1. Label selection.

# You can double-click (Button 1) any label to set the selection to the
# label's text. If a label can take focus (with the -takefocus option),
# a single click (Button 1) will focus the label and then you can Cut,
# Copy and Paste with <Control-x>, <Control-c> and <Control-v>.

# ======================================================================

  namespace eval label {}

# ======================================================================

# pkginit_ --

proc label::pkginit_ {} {

# ----------------------------------------------------------------------

  variable ns [namespace current]

# ----------------------------------------------------------------------

  variable debug ; if { ![info exists debug] } { set debug 1 }

  if { [catch {package require debug} version] } {
proc debug {level string} {
  variable debug
  if { $level <= $debug } { puts -nonewline stderr "label: $string" }
  return
}
  }

  debug 2 "proc pkginit_\n"

# ----------------------------------------------------------------------

  if { [catch {file normalize [info script]} scr] } {
    debug 1 "\[$::errorCode\] $scr\n"
    set scr [file join [pwd] [info script]]
  }
  set tcldir [file dirname $scr]
  set dir [file dirname $tcldir]

  debug 3 "  script = \[$scr\]\n"

# ----------------------------------------------------------------------

  package require -exact Tk $::tcl_version

# ----------------------------------------------------------------------

  variable pkginfo ; array set pkginfo [list \
    package      label \
    version      0.0.1 \
    script       $scr \
    directory    $dir \
    name        "Label" \
    description "A library of label routines." \
    copyright   "Copyright (C) 2003" \
    author      "Mark G. Saye" \
    email       "markgsaye@yahoo.com" \
    date        "April 20, 2003" \
  ]

# ----------------------------------------------------------------------

  bind Label <FocusIn>   [list ${ns}::focus:in  %W]
  bind Label <FocusOut>  [list ${ns}::focus:out %W]

  bind Label <<Copy>>    [list ${ns}::copy  %W]
  bind Label <<Cut>>     [list ${ns}::cut   %W]
  bind Label <<Paste>>   [list ${ns}::paste %W]

  bind Label <Control-c> [list event generate %W <<Copy>>]
  bind Label <Control-v> [list event generate %W <<Paste>>]
  bind Label <Control-x> [list event generate %W <<Cut>>]

  bind Label <Double-1>  [list ${ns}::select %W]
  bind Label <Button-1>  [list ${ns}::button1 %W]

# ----------------------------------------------------------------------

  if { [string equal ::$pkginfo(package) $ns] } {
    package provide $pkginfo(package) $pkginfo(version)
  }

# ----------------------------------------------------------------------

  return

}

# ======================================================================

# button1 --

proc label::button1 {W} {

  if { ![string equal [$W cget -state] disabled] } { focus $W }

# ----------------------------------------------------------------------

  return

}

# ======================================================================

# copy --

proc label::copy {W} {

  debug 2 "proc copy \[$W\]\n"

# ----------------------------------------------------------------------

  clipboard clear
  clipboard append [$W cget -text]

# ----------------------------------------------------------------------

  return

}

# ======================================================================

# cut --

proc label::cut {W} {

  debug 2 "proc cut \[$W\]\n"

# ----------------------------------------------------------------------

  copy $W

  if { ![string equal [$W cget -state] disabled] } {
    $W configure -text ""
  }

# ----------------------------------------------------------------------

  return

}

# ======================================================================

# focus:in --

proc label::focus:in {W} {

  debug 2 "proc focus:in  \[$W\]\n"

# ----------------------------------------------------------------------

  return

}

# ======================================================================

# focus:out --

proc label::focus:out {W} {

  debug 2 "proc focus:out \[$W\]\n"

# ----------------------------------------------------------------------

# if { [string equal [$W cget -state] active] } {
#   $W configure -state normal
# }

# selection handle -type UTF8_STRING $W {}
# selection handle                   $W {}

# ----------------------------------------------------------------------

  return

}

# ======================================================================

# handle --

proc label::handle {W offset maxChars} {

  debug 2 "proc handle \[$W\] \[$offset\] \[$maxChars\]\n"

# ----------------------------------------------------------------------

  return \
    [string range [$W cget -text] $offset [expr {$offset+$maxChars-1}]]

}

# ======================================================================

# normal --

proc label::normal {W} {

  debug 2 "proc normal \[$W\]\n"

# ----------------------------------------------------------------------

  # label has lost the selection - reset state to normal
  if { [string equal [$W cget -state] active] } {
    $W configure -state normal
  }

# ----------------------------------------------------------------------

  return

}

# ======================================================================

# paste --

proc label::paste {W} {

  debug 2 "proc paste \[$W\]\n"

# ----------------------------------------------------------------------

  if { [catch {clipboard get -displayof $W} clip] } {
    puts stderr "\[$::errorCode\] $clip"
    return
  }

  if { ![string equal [$W cget -state] disabled] } {
    $W configure -text $clip
  }

# ----------------------------------------------------------------------

  return

}

# ======================================================================

# select --

proc label::select {W} {

  debug 2 "proc select \[$W\]\n"

# ----------------------------------------------------------------------

  variable ns

# ----------------------------------------------------------------------

  if { [string equal [$W cget -state] disabled] } { return }

  $W configure -state active

  selection own -command [list label::normal $W] $W

  selection handle -type UTF8_STRING $W [list ${ns}::handle $W]
  selection handle                   $W [list ${ns}::handle $W]

# ----------------------------------------------------------------------

  return

}

# ======================================================================

# wrap --

proc label::wrap {W w} {

  debug 2 "proc wrap \[$W\] \[$w\]\n"

# ----------------------------------------------------------------------

  set px [$W cget -padx]

  if { [catch {$W cget -compound} side] } {
    set wl [expr {$w - (2 * $px)}]
  } else {
    switch -- $side {
      left -
      right {
        set image [$W cget -image]
        if { [string length $image] } {
          set iw [image width $image]
        } else {
          set iw 0
        }
        set wl [expr {$w - (3 * $px) - $iw}]
      }
      default {
        set wl [expr {$w - (2 * $px)}]
      }
    }
  }

  $W configure -wraplength $wl

# ----------------------------------------------------------------------

  return

}

# ======================================================================

  label::pkginit_

# ======================================================================

